#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>

#define MB (1024 * 1024)

#define GetVersion		0x00000001
#define GetDeviceID		0x00000003
#define GetDeviceMsg		0x00000005

#define CH37X_BULK0_UP 		0x80000000
#define CH37X_BULK1_UP 		0x80000001
#define CH37X_BULK2_UP 		0x80000002
#define CH37X_BULK3_UP 		0x80000003

#define CH37X_BULK0_DOWN 	0x80000004
#define CH37X_BULK1_DOWN 	0x80000005
#define CH37X_BULK2_DOWN 	0x80000006
#define CH37X_BULK3_DOWN 	0x80000007

#define CH37X_INT0_UP 		0x80000008
#define CH37X_INT1_UP 		0x80000009
#define CH37X_INT2_UP 		0x8000000a
#define CH37X_INT3_UP 		0x8000000b

#define CH37X_INT0_DOWN 	0x8000000c
#define CH37X_INT1_DOWN 	0x8000000d
#define CH37X_INT2_DOWN 	0x8000000e
#define CH37X_INT3_DOWN 	0x8000000f

#define path 	"/dev/ch37x0"
#define BULKLEN	512
#define INTLEN	64

//#define DEBUG

char choice[16];
char choicenum;
int dev_fd;
static unsigned char bulk_in_num = 0;
static unsigned char bulk_out_num = 0;
static unsigned char interrupt_in_num = 0;
static unsigned char interrupt_out_num = 0;

unsigned char bulk_w_buf[BULKLEN];
unsigned char bulk_r_buf[8*BULKLEN];
unsigned char int_w_buf[BULKLEN];
unsigned char int_r_buf[8*BULKLEN];

static void set_random_buffer(unsigned char *buf, int count)
{
	unsigned char val = 0;
	int i = 0;
	struct timeval tval;

	for( i = 0; i < count; i++ ) {
		gettimeofday(&tval, NULL);
		srand(tval.tv_usec);
		*(buf + i) = (unsigned char)rand();
	}
}

static int get_random_value(int max, int min)
{
	int val = 0;
	struct timeval tval;

	if(max < min) {
		printf("input error!\n");
		return -1;
	}
	gettimeofday(&tval, NULL);
	srand(tval.tv_usec);
	val = rand() % (max - min) + min;
	return val;
}

void Show_DevMsg(void)
{
	char *version = malloc(sizeof(char) * 30);
	unsigned long dev_id;
	unsigned long ep_num;
	printf("\n");
	printf("Information of %s lists below:\n", path);
	printf("**************************************\n");
	ioctl(dev_fd, GetVersion, (unsigned long)version);
	ioctl(dev_fd, GetDeviceID, &dev_id);
	ioctl(dev_fd, GetDeviceMsg, &ep_num);
	bulk_in_num = (char)(ep_num>>24);
	bulk_out_num = (char)(ep_num>>16);
	interrupt_in_num = (char)(ep_num>>8);
	interrupt_out_num = (char)ep_num;

	printf("Driver version: %s\n", version);
	printf("Vendor ID: 0x%x, Product ID: 0x%x\n", (unsigned short)dev_id, (unsigned short)(dev_id>>16));
	printf("bulk_in_num: %d\n", bulk_in_num);
	printf("bulk_out_num: %d\n", bulk_out_num);
	printf("interrupt_in_num: %d\n", interrupt_in_num);
	printf("interrupt_out_num: %d\n", interrupt_out_num);
}

static int Bulk_Up(int num)
{
	int retval = 0;
	int Len = sizeof(bulk_r_buf);
	int i = 0;
	int bytes_read = 0;
	
	struct {
		unsigned long pLength;
		unsigned char *r_buf;
	}BulkUp;

	BulkUp.pLength = (unsigned long)&Len;
	BulkUp.r_buf = bulk_r_buf;

	switch(num) {
	case 1:
		if(bulk_in_num >= 1) {
		   printf("Bulk0 data up...\n");
		   retval = ioctl(dev_fd, CH37X_BULK0_UP, (unsigned long)&BulkUp);
		   if(retval >= 0)
			   bytes_read = retval;
		} else 
			printf("Bulk_In_Ep0 Invalid!\n");
		break;
	case 5:
		if(bulk_in_num >= 2) {
		   printf("Bulk1 data up...\n");
		   retval = ioctl(dev_fd, CH37X_BULK1_UP, (unsigned long)&BulkUp);
		   if(retval >= 0)
			   bytes_read = retval;
		} else 
			printf("Bulk_In_Ep1 Invalid!\n");
		break;
	case 9:
		if(bulk_in_num >= 3) {
		   printf("Bulk2 data up...\n");
		   retval = ioctl(dev_fd, CH37X_BULK2_UP, (unsigned long)&BulkUp);
		   if(retval >= 0)
			   bytes_read = retval;
		} else 
			printf("Bulk_In_Ep2 Invalid!\n");
		break;
	case 13:
		if(bulk_in_num >= 4) {
		   printf("Bulk3 data up...\n");
		   retval = ioctl(dev_fd, CH37X_BULK3_UP, (unsigned long)&BulkUp);
		   if(retval >= 0)
			   bytes_read = retval;
		} else 
			printf("Bulk_In_Ep3 Invalid!\n");
		break;
	}

	if(retval < 0) {
	   printf("ioctl error!\n");
	   exit(1);
	}

#ifdef DEBUG
    printf("Buffer read: %d\n", bytes_read);
	for( i = 0; i < bytes_read; i++ )
	   printf("0x%.2x ", bulk_r_buf[i]);
    printf("\n");
#endif

	for( i = 0; i < bytes_read; i++ ) {
		if(bulk_r_buf[i] != (i % 256)) {
			printf("compare error!\n");
			exit(1);
		}
	}

	return retval;
}

static void Speed_Test_Read(int num, int MB_Num)
{
	unsigned long bytes_read = 0;
	struct timeval t1, t2;
	int delta_sec = 0;
	int delta_usec = 0;
	float speed_read = 0.0;

recycle:
	bytes_read = 0;
	gettimeofday(&t1, NULL);
	while(bytes_read <= MB_Num * MB) 
		bytes_read += Bulk_Up(num);
	gettimeofday(&t2, NULL);
	
	delta_sec = t2.tv_sec - t1.tv_sec; 
	delta_usec = t2.tv_usec - t1.tv_usec;

	if(delta_usec < 0) {
		delta_sec --;
		delta_usec = 1000000 + delta_usec;
	}
	speed_read = (float)(MB_Num) / (float)(delta_sec + (float)delta_usec / 1000000); 

	printf("%d MB takes: %2d sec %2d us, up speed: %4.2f MB/s\n", MB_Num, delta_sec, delta_usec, speed_read);

	goto recycle;
}

static int Bulk_Down(int num)
{
	int retval = 0;
	int i = 0;
	int Len = sizeof(bulk_w_buf);
	int bytes_write = 0;

	struct {
		unsigned long pLength;
		unsigned char *w_buf;
	}BulkDown;

//	set_random_buffer(bulk_w_buf, Len);
	
	BulkDown.pLength = (unsigned long)&Len;
	BulkDown.w_buf = bulk_w_buf;

	switch(num) {
	case 2:
		if(bulk_out_num >= 1) {
			printf("Bulk0 data down...\n");
			retval = ioctl(dev_fd, CH37X_BULK0_DOWN, (unsigned long)&BulkDown);
			if(retval >= 0)
				bytes_write = retval;
		} else 
			printf("Bulk_out_Ep0 Invalid!\n");
		break;
	case 6:
		if(bulk_out_num >= 2) {
			printf("Bulk1 data down...\n");
			retval = ioctl(dev_fd, CH37X_BULK1_DOWN, (unsigned long)&BulkDown);
			if(retval >= 0)
				bytes_write = retval;
		} else
			printf("Bulk_out_Ep1 Invalid!\n");
		break;
	case 10:
		if(bulk_out_num >= 3) {
			printf("Bulk2 data down...\n");
			retval = ioctl(dev_fd, CH37X_BULK2_DOWN, (unsigned long)&BulkDown);
			if(retval >= 0)
				bytes_write = retval;
		} else
			printf("Bulk_out_Ep2 Invalid!\n");
		break;
	case 14:
		if(bulk_out_num >= 4) {
			printf("Bulk3 data down...\n");
			retval = ioctl(dev_fd, CH37X_BULK3_DOWN, (unsigned long)&BulkDown);
			if(retval >= 0)
				bytes_write = retval;
		} else
			printf("Bulk_out_Ep3 Invalid!\n");
		break;
	default:
		break;
	}
	if(retval < 0) {
		printf("ioctl error!\n");
		exit(1);
	}
#ifdef DEBUG
	printf("Buffer write: \n");
	for( i = 0; i < Len; i++ )
		printf("0x%x ", bulk_w_buf[i]);
	printf("\n");
#endif
	return retval;
}

static void Speed_Test_Write(int num, int MB_Num)
{
	unsigned long bytes_write = 0;
	struct timeval t1, t2;
	int delta_sec = 0;
	int delta_usec = 0;
	float speed_write = 0.0;

recycle:
	bytes_write = 0;
	gettimeofday(&t1, NULL);
	while(bytes_write <= MB_Num * MB) 
		bytes_write += Bulk_Down(num);
	gettimeofday(&t2, NULL);
	
	delta_sec = t2.tv_sec - t1.tv_sec; 
	delta_usec = t2.tv_usec - t1.tv_usec;

	if(delta_usec < 0) {
		delta_sec --;
		delta_usec = 1000000 + delta_usec;
	}
	speed_write = (float)(MB_Num) / (float)(delta_sec + (float)delta_usec / 1000000); 

	printf("%d MB takes: %2d sec %2d us, down speed: %4.2f MB/s\n", MB_Num, delta_sec, delta_usec, speed_write);

	goto recycle;
}


static int Int_Up(int num)
{
	int retval = 0;
	int i = 0;
	int Len = INTLEN;
	int bytes_read = 0;

	struct {
		unsigned long pLength;
		unsigned char *r_buf;
	}INTUp;

	INTUp.pLength = (unsigned long)&Len;
	INTUp.r_buf = int_r_buf;

	switch(num) {
	case 3:
		printf("INT0 data up...\n");
		retval = ioctl(dev_fd, CH37X_INT0_UP, (unsigned long)&INTUp);
		if(retval >= 0) 
			bytes_read = retval;
		break;
	case 7:
		printf("INT1 data up...\n");
		retval = ioctl(dev_fd, CH37X_INT1_UP, (unsigned long)&INTUp);
		if(retval >= 0) 
			bytes_read = retval;
		break;
	case 11:
		printf("INT2 data up...\n");
		retval = ioctl(dev_fd, CH37X_INT2_UP, (unsigned long)&INTUp);
		if(retval >= 0) 
			bytes_read = retval;
		break;
	case 15:
		printf("INT3 data up...\n");
		retval = ioctl(dev_fd, CH37X_INT3_UP, (unsigned long)&INTUp);
		if(retval >= 0) 
			bytes_read = retval;
		break;
	}
	if(retval < 0) {
		printf("ioctl error!\n");
		exit(1);
	}

#ifdef DEBUG
	printf("Buffer read:\n");
	for( i = 0; i < Len; i++ )
		printf("0x%x ", int_r_buf[i]);
	printf("\n");
#endif
	return retval;
}

static int Int_Down(int num)
{
	int retval = 0;
	int i = 0;
	int Len = sizeof(int_w_buf);
	int bytes_write = 0;
	
	struct {
		unsigned long pLength;
		unsigned char *w_buf;
	}IntDown;

	IntDown.pLength = (unsigned long)&Len;
	IntDown.w_buf = int_w_buf;

	set_random_buffer(int_w_buf, Len);

	switch(num) {
	case 4:
		printf("Int0 data down...\n");
		retval = ioctl(dev_fd, CH37X_INT0_DOWN, (unsigned long)&IntDown);
		if(retval >= 0)
			bytes_write = retval;
		break;
	case 8:
		printf("Int1 data down...\n");
		retval = ioctl(dev_fd, CH37X_INT1_DOWN, (unsigned long)&IntDown);
		if(retval >= 0)
			bytes_write = retval;
		break;
	case 12:
		printf("Int2 data down...\n");
		retval = ioctl(dev_fd, CH37X_INT2_DOWN, (unsigned long)&IntDown);
		if(retval >= 0)
			bytes_write = retval;
		break;
	case 16:
		printf("Int3 data down...\n");
		retval = ioctl(dev_fd, CH37X_INT3_DOWN, (unsigned long)&IntDown);
		if(retval >= 0)
			bytes_write = retval;
		break;
	default:
		printf("Invalid ep number!\n");
		break;
	}
	if(retval < 0) {
		printf("ioctl error!\n");
		exit(1);
	}
#ifdef DEBUG
	printf("Buffer write:\n");
	for( i = 0; i < Len; i++ ) 
		printf("0x%.2x ", int_w_buf[i]);
	printf("\n");
#endif
	return retval;
}

void Show_Choice(void)
{
	int i = 0;
	printf("************************\n");
	printf("0-->Exit\n");
	for( i = 0; i < choicenum; i++ ) {
		switch(choice[i]) {
		case 1: 
			printf("1-->BULK0_UP\n");
			break;
		case 2:
			printf("2-->BULK0_DOWN\n");
			break;
		case 3:
			printf("3-->INT0_UP\n");
			break;
		case 4:
			printf("4-->INT0_DOWN\n");
			break;
		case 5:
			printf("5-->BULK1_UP\n");
			break;
		case 6:
			printf("6-->BULK1_DOWN\n");
			break;
		case 7:
			printf("7-->INT1_UP\n");
			break;
		case 8:
			printf("8-->INT1_DOWN\n");
			break;
		case 9:
			printf("9-->BULK2_UP\n");
			break;
		case 10:
			printf("10-->BULK2_DOWN\n");
			break;
		case 11:
			printf("11-->INT2_UP\n");
			break;
		case 12:
			printf("12-->INT2_DOWN\n");
			break;
		case 13:
			printf("13-->BULK3_UP\n");
			break;
		case 14:
			printf("14-->BULK3_DOWN\n");
			break;
		case 15:
			printf("15-->INT3_UP\n");
			break;
		case 16:
			printf("16-->INT3_DOWN\n");
			break;
		default:
			break;
		}
	}
}

void Show_Menu(void)
{
	char i = 0;
	choicenum = 0;
	memset(choice, 0, sizeof(choice));	
	if(bulk_in_num >= 1) {
		choice[i++] = 1; 
		if(bulk_in_num >= 2) {
			choice[i++] = 5;
			if(bulk_in_num >= 3) {
				choice[i++] = 9;
				if(bulk_in_num >= 4)
					choice[i++] = 13;
			}
		}
	}
	if(bulk_out_num >= 1) {
		choice[i++] = 2; 
		if(bulk_out_num >= 2) {
			choice[i++] = 6;
			if(bulk_out_num >= 3) {
				choice[i++] = 10;
				if(bulk_out_num >= 4)
					choice[i++] = 14;
			}
		}
	}
	if(interrupt_in_num >= 1) {
		choice[i++] = 3; 
		if(interrupt_in_num >= 2) {
			choice[i++] = 7;
			if(interrupt_in_num >= 3) {
				choice[i++] = 10;
				if(interrupt_in_num >= 4)
					choice[i++] = 14;
			}
		}
	}
	if(interrupt_out_num >= 1) {
		choice[i++] = 4; 
		if(interrupt_out_num >= 2) {
			choice[i++] = 8;
			if(interrupt_out_num >= 3) {
				choice[i++] = 11;
				if(interrupt_out_num >= 4)
					choice[i++] = 15;
			}
		}
	}
	choicenum = i;
	Show_Choice();
}

static char test_num(const char *buf, char len, int num)
{
	char i = 0;
	
	for( i = 0; i < len; i++ ) {
		if(num == *(buf + i))
			return 1;
	}
	return 0;
}

void menu(void)
{
	int num = 0;
	int i = 0;
	Show_Menu();
	while(1) {
		printf("************************\n");
		printf("Please input number for test:\n");
		scanf("%d", &num);
		if(!test_num(choice, choicenum, num) && (num != 0)) {
			printf("Bad choice, try again!\n");
			Show_Choice();
			continue;
		}
		switch (num) {
		case 0:
			printf("Test over!\n");
			return;
		case 1:
		case 5:
		case 9:
		case 13:
			Bulk_Up(num);
			break;
		case 2:
		case 6:
		case 10:
		case 14:
			Bulk_Down(num);
			break;
		case 3:
		case 7:
		case 11:
		case 15:
			Int_Up(num);
			break;
		case 4:
		case 8:
		case 12:
		case 16:
			Int_Down(num);
			break;
		default:
			printf("Bad choice, try again!\n");
			break;
		}
	}
}

int main(int argc, char *argv[])
{
	dev_fd = open(path, O_RDWR);
	if(dev_fd < 0) {
		perror("open device fail!\n");
		exit(1);
	}

	Show_DevMsg();
	sleep(3);
//	Speed_Test_Read(1, 100);
//	Speed_Test_Write(2, 100);
	menu();
	close(dev_fd);
	exit(0);
}


